/*
    Copyright 2007 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <windows.h>

typedef uint8_t     u8;
typedef uint16_t    u16;
typedef uint32_t    u32;



#define VER     "0.1"
#define BUFFSZ  256

#define R2KEY1  "\x1F\x48\xF7\xBD\x77\xC3\xA6\xA4\x31\xA9\xC4\xD6\xE1\x42\xF4\xB7"
#define R2KEY2  "\xAF\x8D\x0E\x94\x57\x1A\xA2\xAD\x25\x9A\x6D\x95\x09\xF7\xC6\x81"



void core_online_decrypt(u8 *pass, int len, u8 *key);
void tea_decrypt(uint32_t *p, uint32_t *keyl);
u32 bswap32(u32 value);
int regkeyr(HKEY hKey, LPCTSTR lpSubKey, LPTSTR lpValueName, char *buff, int len);



int main(void) {
    u32     n,
            *r;
    int     i,
            len;
    u8      buff[BUFFSZ],
            path[BUFFSZ],
            key[16],
            *b,
            *k;

    fputs("\n"
        "Toca Race Driver series accounts passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    regkeyr(
        HKEY_LOCAL_MACHINE,
        "SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
        "ProductId",
        buff,
        sizeof(buff));

    b = buff;
    k = key;
    r = (u32 *)R2KEY1;

    for(i = 0; i < 4; i++) {
        n = (b[4] << 24) | (b[2] << 16) | (b[1] << 8) | b[0];
        *(u32 *)k = n ^ r[i];
        k += 4;
        b += 6;
    }

    for(i = 0; ; i++) {
        sprintf(path, "Software\\Codemasters\\Core_Online\\User_%02d", i);

        if(regkeyr(
            HKEY_LOCAL_MACHINE,
            path,
            "Username",
            buff,
            sizeof(buff)) < 0) break;

        printf("\n"
            "- account %02d:\n"
            "  username   %s\n",
            i, buff);

        len = regkeyr(
            HKEY_LOCAL_MACHINE,
            path,
            "Passcode",
            buff,
            sizeof(buff));
        if(len < 0) continue;

        core_online_decrypt(buff, len, key);
        printf("  password   %s\n", buff);
    }

    printf("\n  press RETURN to exit\n");
    fgetc(stdin);
    return(0);
}



void core_online_decrypt(u8 *pass, int len, u8 *key) {
    u32     *p;

    p = (u32 *)pass;
    for(len >>= 3; len; len--) {
        p[0] = bswap32(p[0]);
        p[1] = bswap32(p[1]);
        tea_decrypt(p, (u32 *)key);
        p[0] = bswap32(p[0]);
        p[1] = bswap32(p[1]);
        p += 2;
    }
}



void tea_decrypt(uint32_t *p, uint32_t *keyl) {
    uint32_t    y,
                z,
                sum,
                a = keyl[0],
                b = keyl[1],
                c = keyl[2],
                d = keyl[3];
    int         i;

    y = p[0];
    z = p[1];
    sum = 0xc6ef3720;

    for(i = 0; i < 32; i++) {
        z -= ((c ^ y) + d) + ((y >> 5) ^ sum) + (y << 4);
        y -= ((a ^ z) + b) + ((z >> 5) ^ sum) + (z << 4);
        sum -= 0x9e3779b9;
    }
    p[0] = y;
    p[1] = z;
}



u32 bswap32(u32 value) {
    return((value & 0xff000000) >> 24) |
          ((value & 0x00ff0000) >>  8) |
          ((value & 0x0000ff00) <<  8) |
          ((value & 0x000000ff) << 24);
}



int regkeyr(HKEY hKey, LPCTSTR lpSubKey, LPTSTR lpValueName, char *buff, int len) {
    HKEY    key;

    buff[0] = 0;

    if(RegOpenKeyEx(hKey, lpSubKey, 0, KEY_READ, &key) != ERROR_SUCCESS) {
        return(-1);
    }
    if(RegQueryValueEx(key, lpValueName, NULL, REG_NONE, buff, (void *)&len) != ERROR_SUCCESS) {
        RegCloseKey(key);
        return(-1);
    }
    RegCloseKey(key);

    buff[len] = 0;
    return(len);
}


